#
# (c) Copyright 2001 by philiKON
# Released unter the terms of the GPL
#
# Please see README.txt for further information
#

import os, random, math
import Acquisition, Globals, AccessControl, OFS


# define defaults for new instances of RoboCage
defaults = { 'lang_file':     'english',
             'lang_sort':     1,
             'num_words':     300,
             'num_links':     3,
             'num_emails':    10,
             'email_domains': [ 'com', 'org', 'net' ]  }

file_path = Globals.package_home( globals() )

punctuation = [ ".", ",", "?", "!" ]
punct_lim = math.log( len( punctuation ) + 1 )



class RoboCage ( Acquisition.Implicit, Globals.Persistent, AccessControl.Role.RoleManager, OFS.SimpleItem.Item ):

  """ RoboCage class  """


  ### Zope specific stuff ###

  meta_type = "RoboCage"
  security = AccessControl.ClassSecurityInfo()

  ## manage tabs and DTML methods
  manage_options = (
    {'label' : 'Settings',    'action' : 'manage_main' },
    {'label' : 'View sample', 'action' : 'index_html'  },
   ) + OFS.SimpleItem.Item.manage_options + AccessControl.Role.RoleManager.manage_options

  manage_main = Globals.DTMLFile( "dtml/form_main", globals() )
  index_html = Globals.DTMLFile( "dtml/index_html", globals() )


  def __init__( self, id, title ):
    """ initialize a RoboCage """
    self.id = id
    self.title = title
    self.settings = defaults
    self.rand_lim = self.calc_rand_lim()


  # make sure that there won't be any subpath traversing (except for manage methods)
  def __before_publishing_traverse__( self, obj, request ):
    try:
      x = request.other['TraversalRequestNameStack'][0][:7]
    except IndexError:
      x = ""
    if x != "manage_":
      request.other['TraversalRequestNameStack'] = []


  def get_langs( self ):
    return os.listdir( file_path + "/lang" );


  def manage_saveTitle( self, title, REQUEST=None ):
    """ Save the title """
    self.title = title
    message = "Title saved."
    return self.manage_main( self, REQUEST, manage_tabs_message=message )


  def manage_saveText( self, lang_file, lang_sort, num_words, num_links, num_emails, REQUEST=None ):
    """ Save text options """
    self.settings['lang_file'] = lang_file
    self.settings['lang_sort'] = lang_sort
    self.settings['num_words'] = num_words
    self.settings['num_links'] = num_links
    self.settings['num_emails'] = num_emails

    self.rand_lim = self.calc_rand_lim()

    self._p_changed = 1    # tell the persistence machinery that object has changed
    if REQUEST is not None:
      message = "Text options saved."
      return self.manage_main( self, REQUEST, manage_tabs_message=message )


  def manage_delEmailDomains( self, email_domains, REQUEST=None ):
    """ Delete top-level email domains from list """

    if email_domains:
      self.settings['email_domains'] = filter( lambda i, email_domains=email_domains: i not in email_domains, self.settings['email_domains'] )
      message = "Domains deleted."
    else:
      message = "No domains selected. None deleted."

    self._p_changed = 1    # tell the persistence machinery that object has changed
    if REQUEST is not None:
      return self.manage_main( self, REQUEST, manage_tabs_message=message )


  def manage_addEmailDomain( self, new_domain, REQUEST=None ):
    """ Add a new email domain """
    print "xxxx"
    if new_domain and new_domain not in self.settings['email_domains']:
      self.settings['email_domains'].append( new_domain )
      message = "Domain added."
    else:
      message = "Empty or already existing domain. No changes made."

    self._p_changed = 1    # tell the persistence machinery that object has changed
    if REQUEST is not None:
      return self.manage_main( self, REQUEST, manage_tabs_message=message )




  ### random text generation ###

  def calc_rand_lim( self ):
    """ calculate the upper limit to random generator """
    f = open( file_path + "/lang/" + self.settings['lang_file'], "r" )
    listlen = len( f.readlines() )

    if self.settings['lang_sort']:
      return math.log( listlen + 1 )
    else:
      return listlen


  def get_frand_item( self ):
    """ get an item out a frequency-ordered list """
    randnum = random.uniform( 0, self.rand_lim )
    return int( math.e ** randnum ) - 1


  def get_rand_item( self ):
    """ get a random item """
    return random.randrange( 0, self.rand_lim )


  def generate_text( self ):
    """ generates random (HTML) text using the above functions """

    if self.settings['lang_sort']:
      f = self.get_frand_item
    else:
      f = self.get_rand_item

    r = "<p>"
    infile = open( file_path + "/lang/" + self.settings['lang_file'] )
    words = infile.readlines()

    # find random places to put links at
    links = []
    for i in range( 0, self.settings['num_links'] ):
      x = random.randrange( 0, self.settings['num_words'] )
      while x in links:
        x = random.randrange( 0, self.settings['num_words'] )
      links.append( x )

    # same with fake email addresses
    emails = []
    for i in range( 0, self.settings['num_emails'] ):
      x = random.randrange( 0, self.settings['num_words'] )
      while x in emails or x in links:
        x = random.randrange( 0, self.settings['num_words'] )
      emails.append( x )


    num_words_sentence = random.randrange( 2, 20 )
    num_sent_paragr = random.randrange( 2, 20 )

    for i in range( 0, self.settings['num_words'] ):
      w = words[ f() ]

      # add links and fake email addresses
      if i in links:
        r = "%s <a href=\"%s\">%s</a> " % ( r, w[:-1], w[:-1] )
      elif i in emails:
        topdomain = random.choice( self.settings['email_domains'] )
        subdomain = words[ f() ]
        r = "%s <a href=\"mailto:%s@%s.%s\">%s</a> " % ( r, w[:-1], subdomain.lower(), topdomain, w[:-1] )
      else:
        r = "%s %s" % ( r, w[:-1] )
      num_words_sentence -= 1

      # add punctuation
      if num_words_sentence == 0:
        randnum = random.uniform( 0, punct_lim )
        punct = punctuation[ int( math.e ** randnum ) - 1 ]

        r = "%s%s " % ( r, punct )
        num_words_sentence = random.randrange( 2, 20 )
        num_sent_paragr -= 1

      # add paragraph
      if num_sent_paragr == 0:
        r = "%s</p>\n\n<p>" % r
        num_sent_paragr = random.randrange( 2, 20 )

    r = "%s%s" % ( r, "</p>" )
    return r





### object constructors ###

# form
form_add = Globals.DTMLFile( "dtml/form_add", globals() )

# add the actual instance
def manage_add( self, id, title="", REQUEST=None ):
  """ Create a RoboCage instance """

  rc_instance = RoboCage( id, title )
  self.Destination()._setObject( id, rc_instance )

  if REQUEST is None:
    return

  try:
    url = self.DestinationURL()
  except:
    url = REQUEST['URL1']
  if REQUEST['submit'] == " Add & Edit ":
    url = "%s/%s" % ( url, id )
  REQUEST.RESPONSE.redirect( url + '/manage_main' )

  return


# finally, initialize the product
Globals.InitializeClass( RoboCage )
